@@ -43,7 +43,7 @@ struct GvcStreamStatusIconPrivate
         GtkWidget      *bar;
         guint           current_icon;
         char           *display_name;
-
+        gboolean        thaw;
 };
 
 enum
@@ -66,21 +66,44 @@ on_adjustment_value_changed (GtkAdjustment *adjustment,
 {
         gdouble volume;
 
+        if (icon->priv->thaw)
+                return;
+
         volume = gtk_adjustment_get_value (adjustment);
-        gvc_mixer_stream_change_volume (icon->priv->mixer_stream,
-                                        (guint)volume);
+
+        /* Only push the volume if it's actually changed */
+        if (gvc_mixer_stream_set_volume(icon->priv->mixer_stream,
+                                    (pa_volume_t) round (volume)) != FALSE) {
+                gvc_mixer_stream_push_volume(icon->priv->mixer_stream);
+        }
+}
+
+static void
+update_dock (GvcStreamStatusIcon *icon)
+{
+        GtkAdjustment *adj;
+        gboolean       is_muted;
+
+        g_return_if_fail (icon);
+
+        adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (icon->priv->bar)));
+
+        icon->priv->thaw = TRUE;
+        gtk_adjustment_set_value (adj,
+                                  gvc_mixer_stream_get_volume (icon->priv->mixer_stream));
+        is_muted = gvc_mixer_stream_get_is_muted (icon->priv->mixer_stream);
+        gvc_channel_bar_set_is_muted (GVC_CHANNEL_BAR (icon->priv->bar), is_muted);
+        icon->priv->thaw = FALSE;
 }
 
 static gboolean
 popup_dock (GvcStreamStatusIcon *icon,
             guint                time)
 {
-        GtkAdjustment *adj;
         GdkRectangle   area;
         GtkOrientation orientation;
         GdkDisplay    *display;
         GdkScreen     *screen;
-        gboolean       is_muted;
         gboolean       res;
         int            x;
         int            y;
@@ -88,11 +111,7 @@ popup_dock (GvcStreamStatusIcon *icon,
         GdkRectangle   monitor;
         GtkRequisition dock_req;
 
-        adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (icon->priv->bar)));
-        gtk_adjustment_set_value (adj,
-                                  gvc_mixer_stream_get_volume (icon->priv->mixer_stream));
-        is_muted = gvc_mixer_stream_get_is_muted (icon->priv->mixer_stream);
-        gvc_channel_bar_set_is_muted (GVC_CHANNEL_BAR (icon->priv->bar), is_muted);
+        update_dock (icon);
 
         screen = gtk_status_icon_get_screen (GTK_STATUS_ICON (icon));
         res = gtk_status_icon_get_geometry (GTK_STATUS_ICON (icon),
@@ -112,6 +131,8 @@ popup_dock (GvcStreamStatusIcon *icon,
         monitor_num = gdk_screen_get_monitor_at_point (screen, area.x, area.y);
         gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
+        gtk_container_foreach (GTK_CONTAINER (icon->priv->dock), 
+                               (GtkCallback) gtk_widget_show_all, NULL);
         gtk_widget_size_request (icon->priv->dock, &dock_req);
 
         if (orientation == GTK_ORIENTATION_VERTICAL) {
@@ -153,7 +174,7 @@ popup_dock (GvcStreamStatusIcon *icon,
 
         if (gdk_pointer_grab (icon->priv->dock->window, TRUE,
                               GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-                              GDK_POINTER_MOTION_MASK, NULL, NULL,
+                              GDK_POINTER_MOTION_MASK | GDK_SCROLL_MASK, NULL, NULL,
                               time)
             != GDK_GRAB_SUCCESS) {
                 gtk_grab_remove (icon->priv->dock);
@@ -260,45 +281,13 @@ on_status_icon_popup_menu (GtkStatusIcon       *status_icon,
                         activate_time);
 }
 
-#if GTK_CHECK_VERSION(2,15,0)
 static gboolean
 on_status_icon_scroll_event (GtkStatusIcon       *status_icon,
                              GdkEventScroll      *event,
                              GvcStreamStatusIcon *icon)
 {
-        GtkAdjustment *adj;
-
-        adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (icon->priv->bar)));
-
-        switch (event->direction) {
-        case GDK_SCROLL_UP:
-        case GDK_SCROLL_DOWN: {
-                gdouble volume;
-
-                volume = gtk_adjustment_get_value (adj);
-
-                if (event->direction == GDK_SCROLL_UP) {
-                        volume += adj->step_increment;
-                        if (volume > adj->upper) {
-                                volume = adj->upper;
-                        }
-                } else {
-                        volume -= adj->step_increment;
-                        if (volume < adj->lower) {
-                                volume = adj->lower;
-                        }
-                }
-
-                gtk_adjustment_set_value (adj, volume);
-                return TRUE;
-        }
-        default:
-                break;
-        }
-
-        return FALSE;
+        return gvc_channel_bar_scroll (GVC_CHANNEL_BAR (icon->priv->bar), event->direction);
 }
-#endif
 
 static void
 gvc_icon_release_grab (GvcStreamStatusIcon *icon,
@@ -402,6 +391,16 @@ on_dock_key_release (GtkWidget           *widget,
         return TRUE;
 }
 
+static gboolean
+on_dock_scroll_event (GtkWidget           *widget,
+                      GdkEventScroll      *event,
+                      GvcStreamStatusIcon *icon)
+{
+        /* Forward event to the status icon */
+        on_status_icon_scroll_event (NULL, event, icon);
+        return TRUE;
+}
+
 static void
 update_icon (GvcStreamStatusIcon *icon)
 {
@@ -444,51 +443,31 @@ update_icon (GvcStreamStatusIcon *icon)
 
         if (is_muted) {
                 markup = g_strdup_printf (
-#if GTK_CHECK_VERSION(2,15,0)
                                           "<b>%s: %s</b>\n<small>%s</small>",
-#else
-                                          "%s: %s\n%s",
-#endif
                                           icon->priv->display_name,
                                           _("Muted"),
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         } else if (can_decibel && (db > PA_DECIBEL_MININFTY)) {
                 markup = g_strdup_printf (
-#if GTK_CHECK_VERSION(2,15,0)
                                           "<b>%s: %.0f%%</b>\n<small>%0.2f dB\n%s</small>",
-#else
-                                          "%s: %.0f%%\n%0.2f dB\n%s",
-#endif
                                           icon->priv->display_name,
                                           100 * (float)volume / PA_VOLUME_NORM,
                                           db,
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         } else if (can_decibel) {
                 markup = g_strdup_printf (
-#if GTK_CHECK_VERSION(2,15,0)
                                           "<b>%s: %.0f%%</b>\n<small>-&#8734; dB\n%s</small>",
-#else
-                                          "%s: %.0f%%\n-&#8734; dB\n%s",
-#endif
                                           icon->priv->display_name,
                                           100 * (float)volume / PA_VOLUME_NORM,
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         } else {
                 markup = g_strdup_printf (
-#if GTK_CHECK_VERSION(2,15,0)
                                           "<b>%s: %.0f%%</b>\n<small>%s</small>",
-#else
-                                          "%s: %.0f%%\n%s",
-#endif
                                           icon->priv->display_name,
                                           100 * (float)volume / PA_VOLUME_NORM,
                                           gvc_mixer_stream_get_description (icon->priv->mixer_stream));
         }
-#if GTK_CHECK_VERSION(2,15,0)
         gtk_status_icon_set_tooltip_markup (GTK_STATUS_ICON (icon), markup);
-#else
-        gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon), markup);
-#endif
         g_free (markup);
 }
 
@@ -510,7 +489,7 @@ on_stream_volume_notify (GObject             *object,
                          GvcStreamStatusIcon *icon)
 {
         update_icon (icon);
-        /* FIXME: update dock too */
+        update_dock (icon);
 }
 
 static void
@@ -519,7 +498,7 @@ on_stream_is_muted_notify (GObject             *object,
                            GvcStreamStatusIcon *icon)
 {
         update_icon (icon);
-        /* FIXME: update dock too */
+        update_dock (icon);
 }
 
 void
@@ -671,6 +650,10 @@ gvc_stream_status_icon_constructor (GType                  type,
                           "key-release-event",
                           G_CALLBACK (on_dock_key_release),
                           icon);
+        g_signal_connect (icon->priv->dock,
+                          "scroll-event",
+                          G_CALLBACK (on_dock_scroll_event),
+                          icon);
         g_signal_connect (icon->priv->dock,
                           "grab-notify",
                           G_CALLBACK (on_dock_grab_notify),
@@ -685,7 +668,6 @@ gvc_stream_status_icon_constructor (GType                  type,
         frame = gtk_frame_new (NULL);
         gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
         gtk_container_add (GTK_CONTAINER (icon->priv->dock), frame);
-        gtk_widget_show (frame);
 
         box = gtk_vbox_new (FALSE, 6);
         gtk_container_set_border_width (GTK_CONTAINER (box), 2);
@@ -790,16 +772,16 @@ gvc_stream_status_icon_init (GvcStreamStatusIcon *icon)
                           "popup-menu",
                           G_CALLBACK (on_status_icon_popup_menu),
                           icon);
-#if GTK_CHECK_VERSION(2,15,0)
         g_signal_connect (icon,
                           "scroll-event",
                           G_CALLBACK (on_status_icon_scroll_event),
                           icon);
-#endif
         g_signal_connect (icon,
                           "notify::visible",
                           G_CALLBACK (on_status_icon_visible_notify),
                           NULL);
+
+        icon->priv->thaw = FALSE;
 }
 
 static void
